// MgEventFilter.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGEVENTFILTER_H
#define MGEVENTFILTER_H


/*! \file MgEventFilter.h
	This file includes the base event filter class. It also includes
	all the event classes. You never construct any of these event classes
	yourself.
*/

#include "SDL_events.h"

#include "MgBeginCode.h"

/*! The event code enum. Used by MgEvents. */
enum MgEventCode
{
	MGE_NOEVENT = SDL_NOEVENT,
	MGE_ACTIVE = SDL_ACTIVEEVENT,
	MGE_KEYDOWN = SDL_KEYDOWN,
	MGE_KEYUP = SDL_KEYUP,
	MGE_MOUSEMOTION = SDL_MOUSEMOTION,
	MGE_MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN,
	MGE_MOUSEBUTTONUP = SDL_MOUSEBUTTONUP,
	MGE_JOYAXISMOTION = SDL_JOYAXISMOTION,
	MGE_JOYBALLMOTION = SDL_JOYBALLMOTION,
	MGE_JOYHATMOTION = SDL_JOYHATMOTION,
	MGE_JOYBUTTONDOWN = SDL_JOYBUTTONDOWN,
	MGE_JOYBUTTONUP = SDL_JOYBUTTONUP,
	MGE_QUIT = SDL_QUIT,
	MGE_SYSWMEVENT = SDL_SYSWMEVENT,
	MGE_RESERVEDA = SDL_EVENT_RESERVEDA,
	MGE_RESERVEDB = SDL_EVENT_RESERVEDB,
	MGE_VIDEORESIZE = SDL_VIDEORESIZE,
	MGE_VIDEOEXPOSE = SDL_VIDEOEXPOSE,
	MGE_RESERVED2 = SDL_EVENT_RESERVED2,
	MGE_RESERVED3 = SDL_EVENT_RESERVED3,
	MGE_RESERVED4 = SDL_EVENT_RESERVED4,
	MGE_RESERVED5 = SDL_EVENT_RESERVED5,
	MGE_RESERVED6 = SDL_EVENT_RESERVED6,
	MGE_RESERVED7 = SDL_EVENT_RESERVED7,
	MGE_USEREVENT = SDL_USEREVENT,
	MGE_NUMEVENTS = SDL_NUMEVENTS
};
	
	
//@{
/*! The mask for the event. Used in MgApp::AddFilter(). */
#define MG_ACTIVEEVENT		SDL_EVENTMASK(SDL_ACTIVEEVENT)
#define MG_KEYDOWNEVENT		SDL_EVENTMASK(SDL_KEYDOWN)
#define MG_KEYUPEVENT		SDL_EVENTMASK(SDL_KEYUP)
#define MG_KEYBOARDEVENT	MG_KEYDOWNEVENT | MG_KEYUPEVENT
#define	MG_MOUSEMOTIONEVENT	SDL_EVENTMASK(SDL_MOUSEMOTION)
#define MG_MOUSEDOWNEVENT	SDL_EVENTMASK(SDL_MOUSEBUTTONDOWN)
#define MG_MOUSEUPEVENT		SDL_EVENTMASK(SDL_MOUSEBUTTONUP)
#define MG_MOUSEEVENT		MG_MOUSEMOTIONEVENT | MG_MOUSEDOWNEVENT | MG_MOUSEUPEVENT
#define	MG_JOYAXISMOTION	SDL_EVENTMASK(SDL_JOYAXISMOTION)
#define	MG_JOYBALLMOTION	SDL_EVENTMASK(SDL_JOYBALLMOTION)
#define	MG_JOYHATMOTION	SDL_EVENTMASK(SDL_JOYHATMOTION)
#define	MG_JOYBUTTONUP		SDL_EVENTMASK(SDL_JOYBUTTONUP)
#define	MG_JOYBUTTONDOWN	SDL_EVENTMASK(SDL_JOYBUTTONDOWN)
#define	MG_JOYEVENT			MG_JOYAXISMOTION | MG_JOYBALLMOTION | MG_JOYHATMOTION | MG_JOYBUTTONUP | MG_JOYBUTTONDOWN
#define	MG_VIDEORESIZE		SDL_EVENTMASK(SDL_VIDEORESIZE)
#define	MG_USEREVENT1		SDL_EVENTMASK(24)		// user events start at 24 and end at 31
#define	MG_USEREVENT2		SDL_EVENTMASK(25)
#define	MG_USEREVENT3		SDL_EVENTMASK(26)
#define	MG_USEREVENT4		SDL_EVENTMASK(27)
#define	MG_USEREVENT5		SDL_EVENTMASK(28)
#define	MG_USEREVENT6		SDL_EVENTMASK(29)
#define	MG_USEREVENT7		SDL_EVENTMASK(30)
#define	MG_USEREVENT8		SDL_EVENTMASK(31)
//@}

/*! Pressed state */
#define MG_PRESSED SDL_PRESSED
/*! Released state */
#define MG_RELEASED SDL_RELEASED
/*! See if the passed mouse button is pressed. */
#define MG_BUTTON(X) (MG_PRESSED<<(X-1))

/*! The activation event. */
class MGDLL MgActiveEvent
{
public:
	/*!	Tells if the state was lost or gained.
		\retval true Focus was gained.
		\retval false Focus was lost.
	*/
	bool Status() const;
	/*! A mask of the focus states. */
	Uint8 State() const;

private:
	MgActiveEvent() {}
	MgActiveEvent(SDL_ActiveEvent & e);
	
	SDL_ActiveEvent fEvent;
	
	friend class MgApp;
};

/*! The keyboard event. */
class MGDLL MgKeyboardEvent
{
public:
	/*! \return MG_PRESSED or MG_RELEASED */
	Uint8 State() const;
	/*! \return The key. */
	SDL_keysym Key() const;

private:
	MgKeyboardEvent() {}
	MgKeyboardEvent(SDL_KeyboardEvent & e);
	
	SDL_KeyboardEvent fEvent;
	friend class MgApp;
	friend class MgKeyboard;
};

/*! The mouse motion event. */
class MGDLL MgMouseMotionEvent
{
public:
	/*! \return The button state. You can see which button is pressed by using the MG_BUTTON macro. */
	Uint8 State() const;
	/*! The x coord of the mouse */
	Uint16 X() const;
	/*! The y coord of the mouse */
	Uint16 Y() const;
	/*! The relative motion in the x dir */
	Sint16 XRel() const;
	/*! The relative motion in the y dir */
	Sint16 YRel() const;

private:
	MgMouseMotionEvent() {}
	MgMouseMotionEvent(SDL_MouseMotionEvent & e);
	
	SDL_MouseMotionEvent fEvent;
	friend class MgApp;
	friend class MgMouse;
};

/*! Button is up. */
#define MG_MOUSEBUTTONUP SDL_MOUSEBUTTONUP
/*! Button is down. */
#define MG_MOUSEBUTTONDOWN SDL_MOUSEBUTTONDOWN

/*! The mouse button event. */
class MGDLL MgMouseButtonEvent
{
public:
	/*! \return MG_MOUSE_BUTTON_* */
	Uint8 Type() const;
	/*! \return The button index. */
	Uint8 Button() const;
	/*! \return MG_PRESSED or MG_RELEASED */
	Uint8 State() const;
	/*! X coord. */
	Uint16 X() const;
	/*! Y coord. */
	Uint16 Y() const;
	
private:
	MgMouseButtonEvent() {}
	MgMouseButtonEvent(SDL_MouseButtonEvent & e);
	
	SDL_MouseButtonEvent fEvent;
	friend class MgApp;
	friend class MgMouse;
};

/*! The joystick axis event. */
class MGDLL MgJoyAxisEvent
{
public:
	/*! \return Joystick index. */
	Uint8 Which() const;
	/*! \return Axis index. */
	Uint8 Axis() const;
	/*! \return The axis value. */
	Sint16 Value() const;

private:
	MgJoyAxisEvent() {}
	MgJoyAxisEvent(SDL_JoyAxisEvent & e);
	
	SDL_JoyAxisEvent fEvent;
	friend class MgApp;
	friend class MgJoystick;
};

/*! The joystick ball event. */
class MGDLL MgJoyBallEvent
{
public:
	/*! \return Joystick index. */
	Uint8 Which() const;
	/*! \return The ball index. */
	Uint8 Ball() const;
	/*! \return Relative motion in the x dir. */
	Sint16 XRel() const;
	/*! \return Relative motion in the y dir. */
	Sint16 YRel() const;
	
private:
	MgJoyBallEvent() {}
	MgJoyBallEvent(SDL_JoyBallEvent & e);
	
	SDL_JoyBallEvent fEvent;
	friend class MgApp;
	friend class MgJoystick;
};

/*! The joystick hat event. */
class MGDLL MgJoyHatEvent
{
public:
	/*! \return Joystick index. */
	Uint8 Which() const;
	/*! \return Hat index. */
	Uint8 Hat() const;
	/*! \return The hat position value.
				8	1	2
				7	0	3
				6	5	4
		Zero means that it is centered.
	*/
	Uint8 Value() const;
	
private:
	MgJoyHatEvent() {}
	MgJoyHatEvent(SDL_JoyHatEvent & e);
	
	SDL_JoyHatEvent fEvent;
	friend class MgApp;
	friend class MgJoystick;
};

/*! The joystick button event. */
class MGDLL MgJoyButtonEvent
{
public:
	/*! \return Joystick index. */
	Uint8 Which() const;
	/*! \return Button index. */
	Uint8 Button() const;
	/*! \return Button state. */
	Uint8 State() const;
	
private:
	MgJoyButtonEvent() {}
	MgJoyButtonEvent(SDL_JoyButtonEvent & e);
	
	SDL_JoyButtonEvent fEvent;
	friend class MgApp;
	friend class MgJoystick;
};

/*! The window resize event. */
class MGDLL MgResizeEvent
{
public:
	/*! New width. */
	int W() const;
	/*! New height. */
	int H() const;

private:
	MgResizeEvent() {}
	MgResizeEvent(SDL_ResizeEvent & e);
	
	SDL_ResizeEvent fEvent;
	friend class MgApp;
};

/*! User event. This event you can construct by yourself.
	The variables in this class are public so that you can
	set and unset as you want.
*/
class MGDLL MgUserEvent
{
public:
	/*! Constructor.
	
		\param type SDL_USEREVENT to SDL_NUMEVENTS - 1.
		\param code A user defined code.
		\param data1/data2 User defined data pointers.
	*/
	MgUserEvent(Uint8 _type, int _code = 0, void * _data1 = NULL,
				void * _data2 = NULL);
	~MgUserEvent();
	
	Uint8 type;
	int code;
	void * data1, * data2;
	
	/*! Adds the event to the queue. */
	bool Send();
	
private:
	MgUserEvent() {}
	MgUserEvent(SDL_UserEvent & e);
	
	SDL_UserEvent fEvent;
	friend class MgApp;
};

/*! This is the meat of the events. Derive from this class and reimplement
	the methods for the events you want to catch.
*/
class MGDLL MgEventFilter
{
public:
	//! Constructor.
	MgEventFilter();
	//! Destructor.
	virtual ~MgEventFilter();
	
	//@{
	/*! Implement this method to catch this event type. If you don't want the event passed
		on to other filters return true. Otherwise return false. Only on rarest of occasions
		would you return true to drop the event. As a matter of fact, many of libMgTk's own
		classes rely on you returning true so that they can catch that event.
	*/
	virtual bool Active(const MgActiveEvent & e);
	virtual bool Keyboard(const MgKeyboardEvent & e);
	virtual bool MouseMotion(const MgMouseMotionEvent & e);
	virtual bool MouseButton(const MgMouseButtonEvent & e);
	virtual bool JoystickAxis(const MgJoyAxisEvent & e);
	virtual bool JoystickBall(const MgJoyBallEvent & e);
	virtual bool JoystickHat(const MgJoyHatEvent & e);
	virtual bool JoystickButton(const MgJoyButtonEvent & e);
	virtual bool UserEvent(const MgUserEvent & e);
	//@}
protected:
};

/*! This class can be used to turn certain events on or off. If the event is
	off, it will not be added to the queue. You do not need to construct
	an object of this type to disable and enable events. These methods
	are static.
*/
class MGDLL MgEvents
{
public:
	//! Constructor.
	MgEvents();
	//! Destructor.
	~MgEvents();
	
	/*! Enabled an event. */
	static void Enable(MgEventCode event);
	/*! Disable an event. */
	static void Disable(MgEventCode event);
	/*! Query the status of an event.
		
		\retval true Event is enabled.
		\retval false Event is disabled.
	 */
	static bool Query(MgEventCode event);

private:
};

#include "MgEndCode.h"
#endif