// MgPixelFormat.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGPIXELFORMAT_H
#define MGPIXELFORMAT_H

#include <SDL_video.h>

/*! \file MgPixelFormat.h
	Contains the declaration for the MgPixelFormat class.
*/	

#include "MgColor.h"

#include "MgBeginCode.h"

class MgPalette;

/*!
	This struct holds the masks for the pixel format.
*/
struct MgPixelMask
{
	MgPixelMask() {}
	MgPixelMask(Uint32 r, Uint32 g, Uint32 b, Uint32 a)
	{
		Rmask = r; Gmask = g; Bmask = b; Amask = a;
	}
	
	Uint32 Rmask;
	Uint32 Gmask;
	Uint32 Bmask;
	Uint32 Amask;
};

/*!
	This struct holds the shifts for the pixel format.
*/
struct MgPixelShift
{
	MgPixelShift() {}
	MgPixelShift(Uint8 r, Uint8 g, Uint8 b, Uint8 a)
	{
		Rshift = r; Gshift = g; Bshift = b; Ashift = a;
	}
	
	Uint8 Rshift;
	Uint8 Gshift;
	Uint8 Bshift;
	Uint8 Ashift;
};

/*!
	This struct holds the losses for the pixel format.
*/
struct MgPixelLoss
{
	MgPixelLoss() {}
	MgPixelLoss(Uint8 r, Uint8 g, Uint8 b, Uint8 a)
	{
		Rloss = r; Gloss = g; Bloss = b; Aloss = a;
	}
	
	Uint8 Rloss;
	Uint8 Gloss;
	Uint8 Bloss;
	Uint8 Aloss;
};

	
/*!
	This class is used to handle the pixel format of MgSurfaces.
*/
class MGDLL MgPixelFormat
{
public:
	//! Constructor.
	MgPixelFormat();
	//! Constructor.
	MgPixelFormat(SDL_PixelFormat * fmt);
	//! Destructor.
	~MgPixelFormat();
	
	//! Returns the palette.
	MgPalette *	Palette() const;
	
	//! Maps the RGBA quad to a pixel color.
	Uint32 GetPixel(MgColor color) const;
	//! Takes a pixel color and takes it apart to a RGBA quad.
	MgColor GetColor(Uint32 pixel) const;
	
	//! Bits per pixel.
	Uint8 BitsPerPixel() const;
	//! Bytes per pixel.
	Uint8 BytesPerPixel() const;
	//! Colorkey.
	Uint32 ColorKey() const;
	
	//! Returns the losses for the pixel format.
	MgPixelLoss Loss() const;
	//! Returns the shifts for the pixel format.
	MgPixelShift Shift() const;
	//! Returns the masks for the pixel format.
	MgPixelMask Mask() const;
	
	//! Makes sure the object is valid.
	bool InitCheck() const;

private:
	SDL_PixelFormat * fFormat;
	MgPalette * fPal;
	
	friend class MgSurface;
	friend class MgPainter;
	friend class MgScreen;
	friend class MgVideoSystem;
};

#include "MgEndCode.h"
	
#endif

