// MgSurface.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGSURFACE_H
#define MGSURFACE_H

#include <SDL_video.h>

/*! \file MgSurface.h
	Contians the MgSurface class.
*/

class MgPainter;
class MgMutex;

#include "MgRect.h"
#include "MgColor.h"
#include "MgPixelFormat.h"
#include "MgPalette.h"
#include "MgVertex.h"

#include "MgBeginCode.h"

/*!
	This is the base class for all drawing operations. Never create an MgSurface
	by yourself! Create an MgPainter!
*/
class MGDLL MgSurface
{
public:
	//! Constructor.
	MgSurface();
	//! Constructor.
	MgSurface(SDL_Surface * surface);
	//! Constructor.
	MgSurface(Uint32 flags, int width, int height, int depth,
				Uint32 Rmask, Uint32 Gmask, Uint32 Bmask,
				Uint32 Amask);
	//! Constructor.
	MgSurface(void * pixels, int width, int height, int depth,
				int pitch, Uint32 Rmask, Uint32 Gmask,
				Uint32 Bmask, Uint32 Amask);
	//! Destructor.
	virtual ~MgSurface();
	
	//! Returns the pixel format of the surface.
	MgPixelFormat * Format() const;
	//! Returns the flags for the surface.
	Uint32 Flags() const;
	//! Returns a pointer to the pixel memory.
	Uint8 * Pixels() const;
	//! Width of the surface.
	int Width() const;
	//! Height of the surface.
	int Height() const;
	//! Pitch of the surface.
	Uint16 Pitch() const;
	//! Returns the current clipping rectangle.
	MgRect ClipRect() const;
	
	//! Clears the surface to the specified color.
	void Clear(MgColor col);
	
	//! Locks the surface.
	bool Lock();
	//! Unlocks the surface.
	bool Unlock();
	
	//! Locks a hardware surface.
	bool LockHardware();
	//! Unlocks a hardware surface.
	bool UnlockHardware();
	
	//! Updates the specified rect.
	void UpdateRect(Sint32 x = 0, Sint32 y = 0, Uint32 w = 0, Uint32 h = 0);
	//@{
	//! Updates the specified rectangles.
	void UpdateRects(SDL_Rect * rects, int numRects);
	void UpdateRects(MgRect * rects, int numRects);
	//@}
	
	//! Updates the palette if you changed it.
	bool ApplyPalette();
	
	//! Sets the colorkey.
	bool SetColorKey(Uint32 flag, Uint32 key);
	//! Set the per-surface alpha.
	bool SetAlpha(Uint32 flag, Uint8 alpha);
	//@{
	//! Set the clip rectangle.
	bool SetClipRect(SDL_Rect & rect);
	bool SetClipRect(MgRect & rect);
	//@}
	
	//@{
	//! Convert the surface to the specified format.
	MgPainter * ConvertSurface(SDL_PixelFormat * fmt, Uint32 flags);
	MgPainter * ConvertSurface(MgPixelFormat * fmt, Uint32 flags);
	//@}
	
	//! Converts the surface to the current display format.
	MgPainter * DisplayFormat();
	//! Converts the surface to the current display format, suitable for fast alpha blitting.
	MgPainter * DisplayFormatAlpha();
	
	//! Tells free the surface on destruction of the MgSurface or not. Any derived classes should reimplement this if they do not want the surface to be able to be freed.
	virtual void FreeOnDelete(bool free);
	
	//@{
	//! Blitting functions.
	bool Blit(MgSurface * src, int x, int y);
	bool Blit(SDL_Surface * src, int x, int y);
	bool Blit(MgSurface * src, SDL_Rect rect);
	bool Blit(SDL_Surface * src, SDL_Rect rect);
	bool Blit(MgSurface * src, SDL_Rect srcrect, SDL_Rect dstrect);
	bool Blit(SDL_Surface * src, SDL_Rect srcrect, SDL_Rect dstrect);
	bool Blit(MgSurface * src, SDL_Rect srcrect, int x, int y);
	bool Blit(SDL_Surface * src, SDL_Rect srcrect, int x, int y);
	//@}
	
	//! Returns true if the object is valid.
	virtual bool InitCheck() const;
	
	Uint32 GetPixel(int x, int y);
	void PutPixel(int x, int y, MgColor col);
	void PutPixel(int x, int y, Uint32 col);
	void StrokeHLine(int x1, int x2, int y, MgColor col);
	void StrokeVLine(int y1, int y2, int x, MgColor col);
	void StrokeLine(int x1, int y1, int x2, int y2, MgColor col);
	void StrokeRect(int x1, int y1, int x2, int y2, MgColor col);
	void StrokeCircle(int x, int y, int radius, MgColor col);
	void StrokeEllipse(int x, int y, int xr, int yr, MgColor col);
	void StrokeTriangle(int x1, int y1, int x2, int y2, int x3, int y3,
						MgColor col);
	void StrokePolygon(MgIVertex * vertices, int numVertices);
	
	void FillRect(int x1, int y1, int x2, int y2, MgColor col);
	void FillCircle(int x, int y, int radius, MgColor col);
	void FillEllipse(int x, int y, int xr, int yr, MgColor col);
	void FillTriangle(int x1, int y1, int x2, int y2, int x3, int y3,
						MgColor col);
	void FillPolygon(MgIVertex * vertices, int n, MgColor color);
	void FillGradient(int x1, int y1, int x2, int y2, MgColor startCol,
						MgColor endCol);
	
	void ShadeHLine(int x1, int x2, int y, MgColor c1, MgColor c2);
	void ShadeVLine(int y1, int y2, int x, MgColor c1, MgColor c2);
	void ShadeLine(int x1, int y1, int x2, int y2, MgColor c1,
						MgColor c2);
	void ShadeRect(int x1, int y1, int x2, int y2, MgColor ul,
						MgColor ur, MgColor dl, MgColor dr);
	void ShadeCircle(int x, int y, int r, MgColor center, MgColor outside);
	void ShadeTriangle(int x1, int y1, int x2, int y2, int x3, int y3,
						MgColor c1, MgColor c2, MgColor c3);
	void ShadePolygon(MgIVertex * vertices, int numVertices);
	void ShadeCircleOutline(int x, int y, int r, MgColor top, MgColor right,
							MgColor bottom, MgColor left);
	void ShadeRectOutline(int x1, int y1, int x2, int y2, MgColor topleft,
							MgColor topright, MgColor bottomright,
							MgColor bottomleft);
	
protected:
	MgPixelFormat * fPixelFormat;
	SDL_Surface * fSurface;
	bool fFree;
	MgMutex * fLocker;
	
	// The destructor calls this.
	void Free();
	
	// pointers to the drawing function.
	Uint32 (*getpixel)(SDL_Surface *, int, int);
	void (*putpixel)(SDL_Surface *, int, int, MgColor &);
	void (*putpixel2)(SDL_Surface *, int, int, Uint32);
	void (*strokehline)(SDL_Surface *, int, int, int, MgColor &);
	void (*strokevline)(SDL_Surface *, int, int, int, MgColor &);
	void (*strokeline)(SDL_Surface *, int, int, int, int, MgColor &);
	void (*strokerect)(SDL_Surface *, int, int, int, int, MgColor &);
	void (*strokecircle)(SDL_Surface *, int, int, int, MgColor &);
	void (*strokeellipse)(SDL_Surface *, int, int, int, int, MgColor &);
	void (*stroketriangle)(SDL_Surface *, int, int, int, int, int, int,
							MgColor &);
	void (*strokepolygon)(SDL_Surface *, MgIVertex *, int);
	
	// blending routines
	void (*shadehline)(SDL_Surface *, int, int, int, MgColor &, MgColor &);
	void (*shadevline)(SDL_Surface *, int, int, int, MgColor &, MgColor &);
	void (*shadeline)(SDL_Surface *, int, int, int, int, MgColor &,
						MgColor &);
	void (*shaderect)(SDL_Surface *, int, int, int, int, MgColor &,
						MgColor &, MgColor &, MgColor &);
	void (*shadecircle)(SDL_Surface *, int, int, int, MgColor &, MgColor &);
	void (*shadetriangle)(SDL_Surface *, int, int, int, int, int, int,
							MgColor &, MgColor &, MgColor &);
	void (*shadepolygon)(SDL_Surface *, MgIVertex *, int);

	void (*shadecircleoutline)(SDL_Surface *, int, int, int, MgColor &,
								MgColor &, MgColor &, MgColor &);
	void (*shaderectoutline)(SDL_Surface * sur, int x1, int y1, int x2, int y2,
								MgColor & topleft, MgColor & topright, MgColor &
								bottomright, MgColor & bottomleft);

	// filling routines
	void (*fillrect)(SDL_Surface *, int, int, int, int, MgColor &);
	void (*fillcircle)(SDL_Surface *, int, int, int, MgColor &);
	void (*fillellipse)(SDL_Surface *, int, int, int, int, MgColor &);
	void (*filltriangle)(SDL_Surface *, int, int, int, int, int, int,
							MgColor &);
	void (*fillpolygon)(SDL_Surface *, MgIVertex *, int n, MgColor & color);
	void (*fillgradient)(SDL_Surface *, int, int, int, int, MgColor &,
							MgColor &);
	//void (*fillgradient2)(SDL_Surface *, int, int, int, int, MgColor &,
	//						MgColor &, MgColor &, MgColor &);

	void InitSurface();	// initializes the drawing function pointers

private:
	friend class MgPalette;
	friend class MgImage;
	friend class MgPainter;
	friend class MgScreen;
	friend class MgFont;
};

#include "MgEndCode.h"

#endif

