// MgUDPNet.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGUDPNET_H
#define MGUDPNET_H

#include "MgNet.h"
#include "MgBeginCode.h"


/*! \file MgUDPNet.h
	This file contains the MgUDPNet class.
*/

/*!
	Use this class for UDP networking.
*/
class MGDLL MgUDPNet : public MgNet
{
public:
	
	/*!
		Default constructor.
	*/
	explicit MgUDPNet(void);
	
	/*!
		Constructor.
		
		\param maxSocketSet.
	*/
	
	MgUDPNet(int maxSocketSet);
	
	/*!
		Destructor.
	*/
	~MgUDPNet(void);
	
	/*!
		Open a UDP network socket on a given port.
   	
   		\param port The port to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(Uint16 port);
	
	/*!
		Open a UDP network socket
   	
   		\param ip The MgNetIPAddress to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetIPAddress *ip);
	
	/*!
		Open a UDP socket connection.
	
		\param sockSet The socket set to add the newly UDP socket.
		\param ip.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetSocketSet *sockSet,MgNetIPAddress *ip);
	
	/*!
		Open a UDP socket connection.
	
		\param sockSet The socket set to add the newly UDP socket.
		\param port.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetSocketSet *sockSet,Uint16 port);
	
	/*!
		Get a socket set.
		
		\retval The socket set.
	*/
	MgNetSocketSet *GetSocketSet(void);
	
	/*! 
		Get the IP address of the remote system associated with the socket.
   		If the socket is a server socket, this function returns NULL.
   		
   		\retval The current MgNetIPAddress.
	*/
	MgNetIPAddress * GetPeerAddress(void);
	
	/*! 
		Get the IP address of the remote system associated with the socket.
   		If the socket is a server socket, this function returns NULL.
   		
   		\param channel. The UDP channel to deal with.
   		\retval The current MgNetIPAddress.
	*/
	MgNetIPAddress * GetPeerAddress(int channel);
	
	/*!
		Bind the address 'address' to the first channel on the UDP socket.
		
		\param ip Address to deal with.
		\retval false if it fails to bind.
		\retval true otherwise.
	*/
	bool Bind(MgNetIPAddress *ip);
	
	/*!
		Bind the address 'address' to the requested channel on the UDP socket.
		
		\param channel. Which channel you want to bind on the socket.
		\param ip Address to deal with.
		\retval false if it fails to bind.
		\retval true otherwise.
	*/
	bool Bind(int channel,MgNetIPAddress *ip);
	
	/*!
		Unbind all addresses from the first channel.
		
	*/
	void Unbind(void);
	
	/*!
		Unbind all addresses from the given channel.
		
	*/
	void Unbind(int channel);
	
	/*!
		Allocate a single UDP datagram.
		
		\param udpPacket.
		\param size The size of the datagram.
		\retval false if the systeme is out of memory, then udpPacket is NULL.
		\retval true otherwise.
	*/
	bool AllocPacket(MgNetUDPPacket *udpPacket,int size);
	
	/*!
		Allocate UDP datagrams.
		
		\param udpPackets.
		\param howmany The number of packets to create.
		\param size The size of the datagram.
		\retval false if the systeme is out of memory, then udpPackets is NULL.
		\retval true otherwise.
	*/
	bool AllocPacket(MgNetUDPPacket **udpPackets,int howmany,int size);
		
	/*!
		Free a single UDP datagram.
		
		\param udpPacket.
	*/
	void FreePacket(MgNetUDPPacket *udpPacket);
	
	/*!
		Free UDP datagrams.
		
		\param udpPackets.
	*/
	void FreePacket(MgNetUDPPacket **udpPackets);
	
	/*!
		Resize one UDP datagram.
		
		\param udpPacket.
		\param newSize.
		\retval
	*/
	int ResizePacket(MgNetUDPPacket *udpPacket,int newSize);
	
	/*!
		Resize all UDP datagrams of a vector.
		
		\param udpPackets.
		\param howmany Number of packets in the vector.
		\param newSize.
	*/
	void ResizePacket(MgNetUDPPacket **udpPackets,int howmany,int newSize);

	/*!
		Send data over the first channel. Here for convenience but shouldn't be called.
   		
   		\param data A pointer to any kind type of data. The data will be cast if possible to MgNetUDPPacket internally.
   		\param len Number of bytes sent over the network. Should be the size of a MgNetUDPPacket structure.
   		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
	*/
	bool Send(void *data, int len);
	
	/*!
		Send a datagram on the channel which is contained int the src member of the datagram.
		
		\param udpPacket. The datagram to be sent.
		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
   	*/
   	bool Send(MgNetUDPPacket *udpPacket);
   	
   	/*!
		Send a datagram on the given channel.
		
		\param udpPacket. The datagram to be sent.
		\param channel specified.
		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
   	*/
   	bool Send(MgNetUDPPacket *udpPacket,int channel);
   	
   	/*!
		Send a vector of datagrams on the channel stored in the src member of each packet.
		
		\param udpPackets. The datagram to be sent.
		\param nbPackets to be sent.
		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
   	*/
   	bool Send(MgNetUDPPacket **udpPackets, int nbPackets);
   	
   	/*!
		Receive data from the non-server socket. It does not block.
		
		\param data A pointer to any kind type of data. Should be MgNetUDPPacket.
		\param maxlen Try to get up to maxlen bytes of data.
		\retval false if it fails.
		\retval true otherwise.
	*/
	bool Receive(void *data,int maxlen);
	
	/*!
		Receive MgNetUDPPacket from the UDP socket. It does not block.
		
		\param udpPacket to store the datagram.
		\retval false if an error occured on the network.
		\retval true otherwise.
	*/
	bool Receive(MgNetUDPPacket *udpPacket);
	
	/*!
		Receive MgNetUDPPacket from the UDP socket. It does not block.
		
		\param udpPacket to store the datagram.
		\param nbReceivedPackets to store how many packets the function has read, 0 if there were no packet.
		\retval false if an error occured on the network.
		\retval true otherwise.
	*/
	bool Receive(MgNetUDPPacket *udpPacket,int *nbReceivedPackets);
	
	/*!
		Receive a vector of pending packets from the UDP socket.
   		The returned packets contain the source address and the channel they arrived
   		on.  If they did not arrive on a bound channel, the the channel will be set
   		to -1.
   		The channels are checked in highest to lowest order, so if an address is
   		bound to multiple channels, the highest channel with the source address
   		bound will be returned. It does not block.
   		
   		\param udpPackets.
   		\retval false if there was an error.
   		\retval true otherwise.
   	*/
   	bool Receive(MgNetUDPPacket **udpPackets);
   	
   	/*!
		Receive a vector of pending packets from the UDP socket.
   		The returned packets contain the source address and the channel they arrived
   		on.  If they did not arrive on a bound channel, the the channel will be set
   		to -1.
   		The channels are checked in highest to lowest order, so if an address is
   		bound to multiple channels, the highest channel with the source address
   		bound will be returned. It does not block.
   		
   		\param udpPackets.
   		\param nbReceivedPackets to store how many packets the function has read, 0 if there were no packet.
   		\retval false if there was an error.
   		\retval true otherwise.
   	*/
   	bool Receive(MgNetUDPPacket **udpPackets,int *nbReceivedPackets);
   	
   	/*!
		Handle the client connection.
		
		\param func The callback function calls to handle the client connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	int HandleClient(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data);
	
	/*!
		Handle the server connection.
		
		\param func The callback function calls to handle the server connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	int HandleServer(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data);
	
	/*!
		Check  for events on a socket set.
		
		\param timeout The time which the system will wait for.
		\retval false If an error occured during the listening.
		\retval true otherwise.
	*/
	bool CheckSocketSet(Uint32 timeout);
	
	/*!
		Close the socket
	*/
	void Close(void);
	
private:
	MgNetUDPSocket * udpSock;
};

#include "MgEndCode.h"

#endif